// +build go1.13

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armcompute

import (
	"context"
	"github.com/Azure/azure-sdk-for-go/sdk/armcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"net/http"
	"net/url"
	"strings"
)

// SharedGalleriesClient contains the methods for the SharedGalleries group.
// Don't use this type directly, use NewSharedGalleriesClient() instead.
type SharedGalleriesClient struct {
	con            *armcore.Connection
	subscriptionID string
}

// NewSharedGalleriesClient creates a new instance of SharedGalleriesClient with the specified values.
func NewSharedGalleriesClient(con *armcore.Connection, subscriptionID string) SharedGalleriesClient {
	return SharedGalleriesClient{con: con, subscriptionID: subscriptionID}
}

// Pipeline returns the pipeline associated with this client.
func (client SharedGalleriesClient) Pipeline() azcore.Pipeline {
	return client.con.Pipeline()
}

// Get - Get a shared gallery by subscription id or tenant id.
func (client SharedGalleriesClient) Get(ctx context.Context, location string, galleryUniqueName string, options *SharedGalleriesGetOptions) (PirSharedGalleryResourceResponse, error) {
	req, err := client.getCreateRequest(ctx, location, galleryUniqueName, options)
	if err != nil {
		return PirSharedGalleryResourceResponse{}, err
	}
	resp, err := client.Pipeline().Do(req)
	if err != nil {
		return PirSharedGalleryResourceResponse{}, err
	}
	if !resp.HasStatusCode(http.StatusOK) {
		return PirSharedGalleryResourceResponse{}, client.getHandleError(resp)
	}
	result, err := client.getHandleResponse(resp)
	if err != nil {
		return PirSharedGalleryResourceResponse{}, err
	}
	return result, nil
}

// getCreateRequest creates the Get request.
func (client SharedGalleriesClient) getCreateRequest(ctx context.Context, location string, galleryUniqueName string, options *SharedGalleriesGetOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/sharedGalleries/{galleryUniqueName}"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{location}", url.PathEscape(location))
	urlPath = strings.ReplaceAll(urlPath, "{galleryUniqueName}", url.PathEscape(galleryUniqueName))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2020-09-30")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// getHandleResponse handles the Get response.
func (client SharedGalleriesClient) getHandleResponse(resp *azcore.Response) (PirSharedGalleryResourceResponse, error) {
	result := PirSharedGalleryResourceResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.PirSharedGalleryResource)
	return result, err
}

// getHandleError handles the Get error response.
func (client SharedGalleriesClient) getHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}

// List - List shared galleries by subscription id or tenant id.
func (client SharedGalleriesClient) List(location string, options *SharedGalleriesListOptions) SharedGalleryListPager {
	return &sharedGalleryListPager{
		pipeline: client.con.Pipeline(),
		requester: func(ctx context.Context) (*azcore.Request, error) {
			return client.listCreateRequest(ctx, location, options)
		},
		responder: client.listHandleResponse,
		errorer:   client.listHandleError,
		advancer: func(ctx context.Context, resp SharedGalleryListResponse) (*azcore.Request, error) {
			return azcore.NewRequest(ctx, http.MethodGet, *resp.SharedGalleryList.NextLink)
		},
		statusCodes: []int{http.StatusOK},
	}
}

// listCreateRequest creates the List request.
func (client SharedGalleriesClient) listCreateRequest(ctx context.Context, location string, options *SharedGalleriesListOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/sharedGalleries"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{location}", url.PathEscape(location))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2020-09-30")
	if options != nil && options.SharedTo != nil {
		query.Set("sharedTo", string(*options.SharedTo))
	}
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// listHandleResponse handles the List response.
func (client SharedGalleriesClient) listHandleResponse(resp *azcore.Response) (SharedGalleryListResponse, error) {
	result := SharedGalleryListResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.SharedGalleryList)
	return result, err
}

// listHandleError handles the List error response.
func (client SharedGalleriesClient) listHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}
