// +build go1.13

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armkeyvault

import (
	"context"
	"github.com/Azure/azure-sdk-for-go/sdk/armcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"net/http"
	"net/url"
	"strings"
)

// KeysClient contains the methods for the Keys group.
// Don't use this type directly, use NewKeysClient() instead.
type KeysClient struct {
	con            *armcore.Connection
	subscriptionID string
}

// NewKeysClient creates a new instance of KeysClient with the specified values.
func NewKeysClient(con *armcore.Connection, subscriptionID string) KeysClient {
	return KeysClient{con: con, subscriptionID: subscriptionID}
}

// Pipeline returns the pipeline associated with this client.
func (client KeysClient) Pipeline() azcore.Pipeline {
	return client.con.Pipeline()
}

// CreateIfNotExist - Creates the first version of a new key if it does not exist. If it already exists, then the existing key is returned without any write
// operations being performed. This API does not create subsequent
// versions, and does not update existing keys.
func (client KeysClient) CreateIfNotExist(ctx context.Context, resourceGroupName string, vaultName string, keyName string, parameters KeyCreateParameters, options *KeysCreateIfNotExistOptions) (KeyResponse, error) {
	req, err := client.createIfNotExistCreateRequest(ctx, resourceGroupName, vaultName, keyName, parameters, options)
	if err != nil {
		return KeyResponse{}, err
	}
	resp, err := client.Pipeline().Do(req)
	if err != nil {
		return KeyResponse{}, err
	}
	if !resp.HasStatusCode(http.StatusOK) {
		return KeyResponse{}, client.createIfNotExistHandleError(resp)
	}
	result, err := client.createIfNotExistHandleResponse(resp)
	if err != nil {
		return KeyResponse{}, err
	}
	return result, nil
}

// createIfNotExistCreateRequest creates the CreateIfNotExist request.
func (client KeysClient) createIfNotExistCreateRequest(ctx context.Context, resourceGroupName string, vaultName string, keyName string, parameters KeyCreateParameters, options *KeysCreateIfNotExistOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.KeyVault/vaults/{vaultName}/keys/{keyName}"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	urlPath = strings.ReplaceAll(urlPath, "{vaultName}", url.PathEscape(vaultName))
	urlPath = strings.ReplaceAll(urlPath, "{keyName}", url.PathEscape(keyName))
	req, err := azcore.NewRequest(ctx, http.MethodPut, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2019-09-01")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, req.MarshalAsJSON(parameters)
}

// createIfNotExistHandleResponse handles the CreateIfNotExist response.
func (client KeysClient) createIfNotExistHandleResponse(resp *azcore.Response) (KeyResponse, error) {
	result := KeyResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.Key)
	return result, err
}

// createIfNotExistHandleError handles the CreateIfNotExist error response.
func (client KeysClient) createIfNotExistHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}

// Get - Gets the current version of the specified key from the specified key vault.
func (client KeysClient) Get(ctx context.Context, resourceGroupName string, vaultName string, keyName string, options *KeysGetOptions) (KeyResponse, error) {
	req, err := client.getCreateRequest(ctx, resourceGroupName, vaultName, keyName, options)
	if err != nil {
		return KeyResponse{}, err
	}
	resp, err := client.Pipeline().Do(req)
	if err != nil {
		return KeyResponse{}, err
	}
	if !resp.HasStatusCode(http.StatusOK) {
		return KeyResponse{}, client.getHandleError(resp)
	}
	result, err := client.getHandleResponse(resp)
	if err != nil {
		return KeyResponse{}, err
	}
	return result, nil
}

// getCreateRequest creates the Get request.
func (client KeysClient) getCreateRequest(ctx context.Context, resourceGroupName string, vaultName string, keyName string, options *KeysGetOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.KeyVault/vaults/{vaultName}/keys/{keyName}"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	urlPath = strings.ReplaceAll(urlPath, "{vaultName}", url.PathEscape(vaultName))
	urlPath = strings.ReplaceAll(urlPath, "{keyName}", url.PathEscape(keyName))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2019-09-01")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// getHandleResponse handles the Get response.
func (client KeysClient) getHandleResponse(resp *azcore.Response) (KeyResponse, error) {
	result := KeyResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.Key)
	return result, err
}

// getHandleError handles the Get error response.
func (client KeysClient) getHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}

// GetVersion - Gets the specified version of the specified key in the specified key vault.
func (client KeysClient) GetVersion(ctx context.Context, resourceGroupName string, vaultName string, keyName string, keyVersion string, options *KeysGetVersionOptions) (KeyResponse, error) {
	req, err := client.getVersionCreateRequest(ctx, resourceGroupName, vaultName, keyName, keyVersion, options)
	if err != nil {
		return KeyResponse{}, err
	}
	resp, err := client.Pipeline().Do(req)
	if err != nil {
		return KeyResponse{}, err
	}
	if !resp.HasStatusCode(http.StatusOK) {
		return KeyResponse{}, client.getVersionHandleError(resp)
	}
	result, err := client.getVersionHandleResponse(resp)
	if err != nil {
		return KeyResponse{}, err
	}
	return result, nil
}

// getVersionCreateRequest creates the GetVersion request.
func (client KeysClient) getVersionCreateRequest(ctx context.Context, resourceGroupName string, vaultName string, keyName string, keyVersion string, options *KeysGetVersionOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.KeyVault/vaults/{vaultName}/keys/{keyName}/versions/{keyVersion}"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	urlPath = strings.ReplaceAll(urlPath, "{vaultName}", url.PathEscape(vaultName))
	urlPath = strings.ReplaceAll(urlPath, "{keyName}", url.PathEscape(keyName))
	urlPath = strings.ReplaceAll(urlPath, "{keyVersion}", url.PathEscape(keyVersion))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2019-09-01")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// getVersionHandleResponse handles the GetVersion response.
func (client KeysClient) getVersionHandleResponse(resp *azcore.Response) (KeyResponse, error) {
	result := KeyResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.Key)
	return result, err
}

// getVersionHandleError handles the GetVersion error response.
func (client KeysClient) getVersionHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}

// List - Lists the keys in the specified key vault.
func (client KeysClient) List(resourceGroupName string, vaultName string, options *KeysListOptions) KeyListResultPager {
	return &keyListResultPager{
		pipeline: client.con.Pipeline(),
		requester: func(ctx context.Context) (*azcore.Request, error) {
			return client.listCreateRequest(ctx, resourceGroupName, vaultName, options)
		},
		responder: client.listHandleResponse,
		errorer:   client.listHandleError,
		advancer: func(ctx context.Context, resp KeyListResultResponse) (*azcore.Request, error) {
			return azcore.NewRequest(ctx, http.MethodGet, *resp.KeyListResult.NextLink)
		},
		statusCodes: []int{http.StatusOK},
	}
}

// listCreateRequest creates the List request.
func (client KeysClient) listCreateRequest(ctx context.Context, resourceGroupName string, vaultName string, options *KeysListOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.KeyVault/vaults/{vaultName}/keys"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	urlPath = strings.ReplaceAll(urlPath, "{vaultName}", url.PathEscape(vaultName))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2019-09-01")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// listHandleResponse handles the List response.
func (client KeysClient) listHandleResponse(resp *azcore.Response) (KeyListResultResponse, error) {
	result := KeyListResultResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.KeyListResult)
	return result, err
}

// listHandleError handles the List error response.
func (client KeysClient) listHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}

// ListVersions - Lists the versions of the specified key in the specified key vault.
func (client KeysClient) ListVersions(resourceGroupName string, vaultName string, keyName string, options *KeysListVersionsOptions) KeyListResultPager {
	return &keyListResultPager{
		pipeline: client.con.Pipeline(),
		requester: func(ctx context.Context) (*azcore.Request, error) {
			return client.listVersionsCreateRequest(ctx, resourceGroupName, vaultName, keyName, options)
		},
		responder: client.listVersionsHandleResponse,
		errorer:   client.listVersionsHandleError,
		advancer: func(ctx context.Context, resp KeyListResultResponse) (*azcore.Request, error) {
			return azcore.NewRequest(ctx, http.MethodGet, *resp.KeyListResult.NextLink)
		},
		statusCodes: []int{http.StatusOK},
	}
}

// listVersionsCreateRequest creates the ListVersions request.
func (client KeysClient) listVersionsCreateRequest(ctx context.Context, resourceGroupName string, vaultName string, keyName string, options *KeysListVersionsOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.KeyVault/vaults/{vaultName}/keys/{keyName}/versions"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	urlPath = strings.ReplaceAll(urlPath, "{resourceGroupName}", url.PathEscape(resourceGroupName))
	urlPath = strings.ReplaceAll(urlPath, "{vaultName}", url.PathEscape(vaultName))
	urlPath = strings.ReplaceAll(urlPath, "{keyName}", url.PathEscape(keyName))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2019-09-01")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// listVersionsHandleResponse handles the ListVersions response.
func (client KeysClient) listVersionsHandleResponse(resp *azcore.Response) (KeyListResultResponse, error) {
	result := KeyListResultResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.KeyListResult)
	return result, err
}

// listVersionsHandleError handles the ListVersions error response.
func (client KeysClient) listVersionsHandleError(resp *azcore.Response) error {
	var err CloudError
	if err := resp.UnmarshalAsJSON(&err); err != nil {
		return err
	}
	return azcore.NewResponseError(&err, resp.Response)
}
