// +build go1.13

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armresources

import (
	"context"
	"github.com/Azure/azure-sdk-for-go/sdk/armcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"net/http"
	"time"
)

// DeploymentExtendedPoller provides polling facilities until the operation completes
type DeploymentExtendedPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (DeploymentExtendedResponse, error)
	ResumeToken() (string, error)
}

type deploymentExtendedPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *deploymentExtendedPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *deploymentExtendedPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *deploymentExtendedPoller) FinalResponse(ctx context.Context) (DeploymentExtendedResponse, error) {
	respType := DeploymentExtendedResponse{DeploymentExtended: &DeploymentExtended{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.DeploymentExtended)
	if err != nil {
		return DeploymentExtendedResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeDeploymentExtendedPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *deploymentExtendedPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *deploymentExtendedPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (DeploymentExtendedResponse, error) {
	respType := DeploymentExtendedResponse{DeploymentExtended: &DeploymentExtended{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.DeploymentExtended)
	if err != nil {
		return DeploymentExtendedResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// DeploymentValidateResultPoller provides polling facilities until the operation completes
type DeploymentValidateResultPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (DeploymentValidateResultResponse, error)
	ResumeToken() (string, error)
}

type deploymentValidateResultPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *deploymentValidateResultPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *deploymentValidateResultPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *deploymentValidateResultPoller) FinalResponse(ctx context.Context) (DeploymentValidateResultResponse, error) {
	respType := DeploymentValidateResultResponse{DeploymentValidateResult: &DeploymentValidateResult{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.DeploymentValidateResult)
	if err != nil {
		return DeploymentValidateResultResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeDeploymentValidateResultPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *deploymentValidateResultPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *deploymentValidateResultPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (DeploymentValidateResultResponse, error) {
	respType := DeploymentValidateResultResponse{DeploymentValidateResult: &DeploymentValidateResult{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.DeploymentValidateResult)
	if err != nil {
		return DeploymentValidateResultResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// GenericResourcePoller provides polling facilities until the operation completes
type GenericResourcePoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (GenericResourceResponse, error)
	ResumeToken() (string, error)
}

type genericResourcePoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *genericResourcePoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *genericResourcePoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *genericResourcePoller) FinalResponse(ctx context.Context) (GenericResourceResponse, error) {
	respType := GenericResourceResponse{GenericResource: &GenericResource{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.GenericResource)
	if err != nil {
		return GenericResourceResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeGenericResourcePoller method
// on the client to create a new poller from the data held in the current poller type
func (p *genericResourcePoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *genericResourcePoller) pollUntilDone(ctx context.Context, frequency time.Duration) (GenericResourceResponse, error) {
	respType := GenericResourceResponse{GenericResource: &GenericResource{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.GenericResource)
	if err != nil {
		return GenericResourceResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// HTTPPoller provides polling facilities until the operation completes
type HTTPPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (*http.Response, error)
	ResumeToken() (string, error)
}

type httpPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *httpPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *httpPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *httpPoller) FinalResponse(ctx context.Context) (*http.Response, error) {
	return p.pt.FinalResponse(ctx, p.pipeline, nil)
}

// ResumeToken generates the string token that can be used with the ResumeHTTPPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *httpPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *httpPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (*http.Response, error) {
	return p.pt.PollUntilDone(ctx, frequency, p.pipeline, nil)
}

// ResourceGroupExportResultPoller provides polling facilities until the operation completes
type ResourceGroupExportResultPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (ResourceGroupExportResultResponse, error)
	ResumeToken() (string, error)
}

type resourceGroupExportResultPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *resourceGroupExportResultPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *resourceGroupExportResultPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *resourceGroupExportResultPoller) FinalResponse(ctx context.Context) (ResourceGroupExportResultResponse, error) {
	respType := ResourceGroupExportResultResponse{ResourceGroupExportResult: &ResourceGroupExportResult{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.ResourceGroupExportResult)
	if err != nil {
		return ResourceGroupExportResultResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeResourceGroupExportResultPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *resourceGroupExportResultPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *resourceGroupExportResultPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (ResourceGroupExportResultResponse, error) {
	respType := ResourceGroupExportResultResponse{ResourceGroupExportResult: &ResourceGroupExportResult{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.ResourceGroupExportResult)
	if err != nil {
		return ResourceGroupExportResultResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// WhatIfOperationResultPoller provides polling facilities until the operation completes
type WhatIfOperationResultPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (WhatIfOperationResultResponse, error)
	ResumeToken() (string, error)
}

type whatIfOperationResultPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *whatIfOperationResultPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *whatIfOperationResultPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *whatIfOperationResultPoller) FinalResponse(ctx context.Context) (WhatIfOperationResultResponse, error) {
	respType := WhatIfOperationResultResponse{WhatIfOperationResult: &WhatIfOperationResult{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.WhatIfOperationResult)
	if err != nil {
		return WhatIfOperationResultResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeWhatIfOperationResultPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *whatIfOperationResultPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *whatIfOperationResultPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (WhatIfOperationResultResponse, error) {
	respType := WhatIfOperationResultResponse{WhatIfOperationResult: &WhatIfOperationResult{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.WhatIfOperationResult)
	if err != nil {
		return WhatIfOperationResultResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}
