// +build go1.13

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armstorage

import (
	"context"
	"github.com/Azure/azure-sdk-for-go/sdk/armcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"net/http"
	"time"
)

// BlobRestoreStatusPoller provides polling facilities until the operation completes
type BlobRestoreStatusPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (BlobRestoreStatusResponse, error)
	ResumeToken() (string, error)
}

type blobRestoreStatusPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *blobRestoreStatusPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *blobRestoreStatusPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *blobRestoreStatusPoller) FinalResponse(ctx context.Context) (BlobRestoreStatusResponse, error) {
	respType := BlobRestoreStatusResponse{BlobRestoreStatus: &BlobRestoreStatus{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.BlobRestoreStatus)
	if err != nil {
		return BlobRestoreStatusResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeBlobRestoreStatusPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *blobRestoreStatusPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *blobRestoreStatusPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (BlobRestoreStatusResponse, error) {
	respType := BlobRestoreStatusResponse{BlobRestoreStatus: &BlobRestoreStatus{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.BlobRestoreStatus)
	if err != nil {
		return BlobRestoreStatusResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// HTTPPoller provides polling facilities until the operation completes
type HTTPPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (*http.Response, error)
	ResumeToken() (string, error)
}

type httpPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *httpPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *httpPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *httpPoller) FinalResponse(ctx context.Context) (*http.Response, error) {
	return p.pt.FinalResponse(ctx, p.pipeline, nil)
}

// ResumeToken generates the string token that can be used with the ResumeHTTPPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *httpPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *httpPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (*http.Response, error) {
	return p.pt.PollUntilDone(ctx, frequency, p.pipeline, nil)
}

// StorageAccountPoller provides polling facilities until the operation completes
type StorageAccountPoller interface {
	Done() bool
	Poll(ctx context.Context) (*http.Response, error)
	FinalResponse(ctx context.Context) (StorageAccountResponse, error)
	ResumeToken() (string, error)
}

type storageAccountPoller struct {
	// the client for making the request
	pipeline azcore.Pipeline
	pt       armcore.Poller
}

// Done returns true if there was an error or polling has reached a terminal state
func (p *storageAccountPoller) Done() bool {
	return p.pt.Done()
}

// Poll will send poll the service endpoint and return an http.Response or error received from the service
func (p *storageAccountPoller) Poll(ctx context.Context) (*http.Response, error) {
	return p.pt.Poll(ctx, p.pipeline)
}

func (p *storageAccountPoller) FinalResponse(ctx context.Context) (StorageAccountResponse, error) {
	respType := StorageAccountResponse{StorageAccount: &StorageAccount{}}
	resp, err := p.pt.FinalResponse(ctx, p.pipeline, respType.StorageAccount)
	if err != nil {
		return StorageAccountResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}

// ResumeToken generates the string token that can be used with the ResumeStorageAccountPoller method
// on the client to create a new poller from the data held in the current poller type
func (p *storageAccountPoller) ResumeToken() (string, error) {
	return p.pt.ResumeToken()
}

func (p *storageAccountPoller) pollUntilDone(ctx context.Context, frequency time.Duration) (StorageAccountResponse, error) {
	respType := StorageAccountResponse{StorageAccount: &StorageAccount{}}
	resp, err := p.pt.PollUntilDone(ctx, frequency, p.pipeline, respType.StorageAccount)
	if err != nil {
		return StorageAccountResponse{}, err
	}
	respType.RawResponse = resp
	return respType, nil
}
