// +build go1.13

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armstorage

import (
	"context"
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/armcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
)

// SKUsClient contains the methods for the SKUs group.
// Don't use this type directly, use NewSKUsClient() instead.
type SKUsClient struct {
	con            *armcore.Connection
	subscriptionID string
}

// NewSKUsClient creates a new instance of SKUsClient with the specified values.
func NewSKUsClient(con *armcore.Connection, subscriptionID string) SKUsClient {
	return SKUsClient{con: con, subscriptionID: subscriptionID}
}

// Pipeline returns the pipeline associated with this client.
func (client SKUsClient) Pipeline() azcore.Pipeline {
	return client.con.Pipeline()
}

// List - Lists the available SKUs supported by Microsoft.Storage for given subscription.
func (client SKUsClient) List(ctx context.Context, options *SKUsListOptions) (StorageSKUListResultResponse, error) {
	req, err := client.listCreateRequest(ctx, options)
	if err != nil {
		return StorageSKUListResultResponse{}, err
	}
	resp, err := client.Pipeline().Do(req)
	if err != nil {
		return StorageSKUListResultResponse{}, err
	}
	if !resp.HasStatusCode(http.StatusOK) {
		return StorageSKUListResultResponse{}, client.listHandleError(resp)
	}
	result, err := client.listHandleResponse(resp)
	if err != nil {
		return StorageSKUListResultResponse{}, err
	}
	return result, nil
}

// listCreateRequest creates the List request.
func (client SKUsClient) listCreateRequest(ctx context.Context, options *SKUsListOptions) (*azcore.Request, error) {
	urlPath := "/subscriptions/{subscriptionId}/providers/Microsoft.Storage/skus"
	urlPath = strings.ReplaceAll(urlPath, "{subscriptionId}", url.PathEscape(client.subscriptionID))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	query := req.URL.Query()
	query.Set("api-version", "2019-06-01")
	req.URL.RawQuery = query.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// listHandleResponse handles the List response.
func (client SKUsClient) listHandleResponse(resp *azcore.Response) (StorageSKUListResultResponse, error) {
	result := StorageSKUListResultResponse{RawResponse: resp.Response}
	err := resp.UnmarshalAsJSON(&result.StorageSKUListResult)
	return result, err
}

// listHandleError handles the List error response.
func (client SKUsClient) listHandleError(resp *azcore.Response) error {
	body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return fmt.Errorf("%s; failed to read response body: %w", resp.Status, err)
	}
	if len(body) == 0 {
		return azcore.NewResponseError(errors.New(resp.Status), resp.Response)
	}
	return azcore.NewResponseError(errors.New(string(body)), resp.Response)
}
